"use strict";
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
var util = __importStar(require("util"));
function timeDiff(start, end) {
    var diff = end - start;
    if (diff < 60000) {
        return diff + "ms";
    }
    else {
        return Math.floor(diff / 1000) + "s";
    }
}
/** A decorator for logging method enter/exit
 *
 * @param logger if it is a string, it is considered as "this[logger]", otherwise, we expect it is the logger object
 * @param opt an object with the following keys
 * @param opt.enterOnly if true, log when calling the function, return value will be ignored
 * @param opt.ignorePromise if true, will NOT track promise resolve/reject
 * @param opt.level a string, the log level, default is "debug"
 * @param opt.errLevel a string, the log level used when logging errors, default is "warn"
 * @param opt.timer if true, will log the total execution time, only valid for promise result
 */
function autolog(logger, opt) {
    return function (target, propertyName, propertyDescriptor) {
        var method = propertyDescriptor.value;
        propertyDescriptor.value = function () {
            var args = [];
            for (var _i = 0; _i < arguments.length; _i++) {
                args[_i] = arguments[_i];
            }
            var inspectOpt = { maxArrayLength: 2, breakLength: 1024 };
            // convert list of greet arguments to string, use Node.util.inspect, not JSON.stringify
            var params = args.map(function (item) { return util.inspect(item, inspectOpt); }).join();
            var typeName = target && target.constructor && target.constructor.name;
            var fullName = (typeName) ? typeName + "." + propertyName : propertyName;
            var log;
            if (typeof logger === "string" || logger instanceof String) {
                // @ts-ignore
                log = this[logger];
            }
            else {
                log = logger;
            }
            // invoke the function() and get its return value
            // tslint:disable-next-line:try-catch-first
            var level = (opt && opt.level) || "debug";
            var errLevel = (opt && opt.errLevel) || "warn";
            if (opt && opt.enterOnly) {
                log.log(level, fullName + "(" + params + ") called");
                return method.apply(this, args);
            }
            // tslint:disable-next-line:try-catch-first
            try {
                var startTime_1 = (opt && opt.timer) ? new Date().getTime() : 0;
                var result = method.apply(this, args);
                if (result instanceof Promise) {
                    log.log(level, "async " + fullName + "(" + params + ") called");
                    if (opt && opt.ignorePromise !== undefined && !opt.ignorePromises) {
                        return result;
                    }
                    var promise = result;
                    return promise.then(function (promiseValue) {
                        var resultStr = util.inspect(promiseValue, inspectOpt);
                        if (startTime_1) {
                            var diff = timeDiff(startTime_1, new Date().getTime());
                            log.log(level, "async " + fullName + "(" + params + ") (" + diff + ") => " + resultStr);
                        }
                        else {
                            log.log(level, "async " + fullName + "(" + params + ") => " + resultStr);
                        }
                        return promiseValue;
                    }).catch(function (err) {
                        if (startTime_1) {
                            var diff = timeDiff(startTime_1, new Date().getTime());
                            log.log(errLevel, "async " + fullName + "(" + params + ") (" + diff + ") => " + err);
                        }
                        else {
                            log.log(errLevel, "async " + fullName + "(" + params + ") => " + err);
                        }
                        throw err;
                    });
                }
                else {
                    var resultStr = util.inspect(result, inspectOpt);
                    log.log(level, fullName + "(" + params + ") => " + resultStr);
                    return result;
                }
            }
            catch (err) {
                log.log(errLevel, fullName + "(" + params + ") => " + err);
                throw err;
            }
        };
        return propertyDescriptor;
    };
}
exports.autolog = autolog;
